<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://meydjer.com
 * @since      1.0.0
 *
 * @package    Lastform
 * @subpackage Lastform/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Lastform
 * @subpackage Lastform/admin
 * @author     Meydjer Windmüller <meydjer@gmail.com>
 */
class Lastform_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $_slug    The ID of this plugin.
	 */
	private $_slug;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $_version    The current version of this plugin.
	 */
	private $_version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $_slug       The name of this plugin.
	 * @param      string    $_version    The version of this plugin.
	 */
	public function __construct( $_slug, $_version ) {

		$this->_slug = $_slug;
		$this->_version = $_version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Lastform_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Lastform_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->_slug, plugin_dir_url( __FILE__ ) . 'css/lastform-admin.css', array(), $this->_version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Lastform_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Lastform_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->_slug, plugin_dir_url( __FILE__ ) . 'js/lastform-admin.js', array( 'jquery' ), $this->_version, false );

	}

}
